;******************************************************************************
;   This file is a basic code template for code generation                    *
;   on the  PIC16F88. This file contains the basic code building              *
;   blocks to build upon.                                                     *
;                                                                             *
;   Refer to the MPASM User's Guide for additional information on             *
;   features of the assembler.                                                *
;                                                                             *
;   Refer to the respective data sheet for additional                         *
;   information on the instruction set.                                       *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Filename:         rainbow9.asm                                           *
;    Date:             23-03-2009                                             *
;    File Version:                                                            *
;                                                                             *
;    Author:           Pete Griffiths                                         *
;    Company:          PIC Projects                                           *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Files Required: P16F88.INC                                               *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Features of the 16F88:                                                   *
;                                                                             *
;    1 10-bit PWM                                                             *
;    8 MHz Internal Oscillator                                                *
;    ICD support                                                              *
;    256 bytes of EEPROM data memory                                          *
;    Capture/Compare Module                                                   *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Notes:                                                                   *
;                                                                             *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Revision History:                                                        *
;                                                                             *
;******************************************************************************

;------------------------------------------------------------------------------
; PROCESSOR DECLARATION
;------------------------------------------------------------------------------

     LIST      p=16F88              ; list directive to define processor
     #INCLUDE <P16F88.INC>          ; processor specific variable definitions

;------------------------------------------------------------------------------
;
; CONFIGURATION WORD SETUP
;
; The 'CONFIG' directive is used to embed the configuration word within the 
; .asm file. The lables following the directive are located in the respective 
; .inc file.  See the data sheet for additional information on configuration 
; word settings.
;
;------------------------------------------------------------------------------

     __CONFIG    _CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_ON & _PWRTE_ON & _WDT_OFF & _INTRC_IO
     __CONFIG    _CONFIG2, _IESO_OFF & _FCMEN_OFF

  
         	errorlevel -302	; suppress banksel warning messages

;------------------------------------------------------------------------------
;
; VARIABLE DEFINITIONS
;
; Available Data Memory divided into Bank 0 through Bank 3.  Each Bank contains
; Special Function Registers and General Purpose Registers at the locations 
; below:  
;
;           SFR           GPR               SHARED GPR's
; Bank 0    0x00-0x1F     0x20-0x6F         0x70-0x7F    
; Bank 1    0x80-0x9F     0xA0-0xEF         0xF0-0xFF  
; Bank 2    0x100-0x10F   0x110-0x16F       0x170-0x17F
; Bank 3    0x180-0x18F   0x190-0x1EF       0x1F0-0x1FF
;
;------------------------------------------------------------------------------

    CBLOCK 0x20 ; Sample GPR variable registers allocated contiguously

	led.red1
	led.grn1
	led.blu1
	led.red2
	led.grn2
	led.blu2
	led.red3
	led.grn3
	led.blu3

	led.red4
	led.grn4
	led.blu4
	led.red5
	led.grn5
	led.blu5
	led.red6
	led.grn6
	led.blu6

	led.red7
	led.grn7
	led.blu7
	led.red8
	led.grn8
	led.blu8
	led.red9
	led.grn9
	led.blu9
	cdata.index
	cdata.master.idx
	rateL
	rateH
	speed
	spread
	direction
	position
	tick




    ENDC

	cblock 0x70
	pwm.period.count
	porta.copy
	portb.copy
	led.group
	flag
	angle.red
	angle.grn
	angle.blu
	W_TEMP	; w register for context saving (ACCESS)
	STATUS_TEMP	; status used for context saving (ACCESS)
	PCLATH_TEMP	; variable used for context saving


	endc

;------------------------------------------------------------------------------
; EEPROM INITIALIZATION
;
; The 16F88 has 256 bytes of non-volatile EEPROM, starting at address 0x2100
; 
;------------------------------------------------------------------------------

DATAEE    ORG  0x2100
    DE    "Rainbow9"  ;

;------------------------------------------------------------------------------
; Contants and equates
;------------------------------------------------------------------------------


led.sel0	equ	.0
led.sel1	equ	.1
led.sel2	equ	.2

led.red.grp1	equ	.0
led.grn.grp1	equ	.1
led.blu.grp1	equ	.2

; flags
cycle	equ	0
timeOut	equ	1
transfer	equ	2
LEDbank	equ	3

;------------------------------------------------------------------------------
; RESET VECTOR
;------------------------------------------------------------------------------

RESET     ORG     0x0000            ; processor reset vector
          PAGESEL START
          GOTO    START             ; go to beginning of program

;------------------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE
;------------------------------------------------------------------------------

ISR       ORG     0x0004            ; interrupt vector location

;------------------------------------------------------------------------------
; USER INTERRUPT SERVICE ROUTINE GOES HERE

	#include rainbow9_inth.inc

;------------------------------------------------------------------------------
; MAIN PROGRAM
;------------------------------------------------------------------------------

START	clrf	STATUS
	clrf	PORTB

	banksel	OSCCON
	movlw	b'01110000'	; set OSCCON for internal RC oscillator
	movwf	OSCCON	; running at 8Mhz
	
	movlw	b'00000000'	; enable weak pull up on PORTB
	movwf	OPTION_REG


	clrf	ANSEL	; Set I/O ports for digital mode

	movlw	b'00000000'
	movwf	TRISB

	movlw	b'11110000'
	movwf	TRISA

	movlw	.125
	movwf	PR2
	bsf	PIE1, TMR2IE	

	banksel	PORTA


	movlw	0x20
	movwf	FSR
clear.gpr	clrf	INDF
	bsf	FSR,7
	clrf	INDF
	bcf	FSR,7
	incf	FSR,F
	btfss	FSR,7
	goto	clear.gpr

	clrf	cdata.master.idx
	call	cdata.entry
	
	movlw	.1
	movwf	led.group
	movwf	direction
	movwf	speed
	movwf	rateH
	movwf	rateL


	movlw	.255/.9
	movwf	spread


	movlw	1<<TMR2ON 
	movwf	T2CON
	bcf	PIR1, TMR2IF
	bsf	INTCON, PEIE
	bsf	INTCON, GIE


;-----------------------------------------------
; Main code here
test.loop	btfss	flag, timeOut
	goto	test.loop
	bcf	flag,timeOut

wait.for.transfer
	btfsc	flag, transfer
	goto	wait.for.transfer

	call	cdata.entry
	bsf	flag, transfer

;
	movlw	.96
	movwf	speed

	movlw	.256/.9
	movwf	spread

	movlw	.0
	movwf	angle.red
	movlw	.255 / .3
	movwf	angle.grn
	movlw	.255 / .3 * .2
	movwf	angle.blu	

	movlw	.1
	addwf	cdata.master.idx,F
	goto	test.loop



;-----------------------------------------------------
; Colour lookup

cdata.entry	movlw	HIGH table.base
	movwf	PCLATH
	movlw	led.red1
	btfss	flag,LEDbank	; if flag set use bank0 else use bank1
	movlw	led.red1 | 1<<7
	movwf	FSR
	movfw	cdata.master.idx
	movwf	cdata.index

cdata.next	call	cdata.read
	movfw	spread
	addwf	cdata.index,F
	movfw	FSR
	andlw	0x7F	; mask off bit 7 
	xorlw	led.blu9+.1
	skpz
	goto	cdata.next
	return

;-----------------------------------------------------
cdata.read	movfw	angle.red
	call	read.table
	movwf	INDF
	incf	FSR,F
	movfw	angle.grn
	call	read.table
	movwf	INDF
	incf	FSR,F
	movfw	angle.blu
	call	read.table
	movwf	INDF
	incf	FSR,F
	
	return


read.table	addwf 	cdata.index,W
	addwf 	PCL,F

	org 0x200
	radix dec
table.base	
	#include sinColTable.inc
	;#include colourTable.inc
	;#include colourTableL1.inc
	#include colourTableS1.inc




          END
