;         Context saving for ISR
          	MOVWF   W_TEMP            ; save off current W register contents
          	MOVF    STATUS,W          ; move status register into W register
          	MOVWF   STATUS_TEMP       ; save off contents of STATUS register
          	MOVF    PCLATH,W          ; move pclath register into W register
          	MOVWF   PCLATH_TEMP       ; save off contents of PCLATH register
          	CLRF    PCLATH            ; Make sure PCLATH points to page 0
	
	banksel	PORTA	; select bank 0 always
	bcf	PIR1, TMR2IF	; clear Timer2 interrupt flag
			; since it's the only interrupt source there
			; is no need to test flag.

clock.timer	comf	tick,F
	skpz
	goto	pwm
	decfsz	rateH,F
	goto	pwm
	movfw	speed
	movwf	rateH
	bsf	flag, timeOut


pwm	incfsz	pwm.period.count,F
	goto	pwm.transfer

	movlw	.255-.64	; using 6 bit PWM so reload count with offset
	movwf	pwm.period.count

	clrf	porta.copy	; clear working variables

		; select next group of LEDs
scan.leds	clrc
	rlf	led.group,W
	btfsc	led.group,2
	movlw	.1
	movwf	led.group

pwm.transfer	btfss	flag, transfer	; test transfer flag
	goto	pwm.duty		; if clear skip block
	bcf	flag, transfer	; clear transfer flag
	movlw	1<<LEDbank		; toggle LED bank flag
	xorwf	flag,F

pwm.duty	btfsc	flag, LEDbank	; if flag set use Bank1 else use Bank0
	bsf	STATUS,RP0	; select bank 1
	
	; LEDs are grouped into 3 groups of 3 RGB LEDs
	; run PWM for group selected by flag bit in led.group variable
	btfsc	led.group,0	
	goto	pwm.source1	
	btfsc	led.group,1
	goto	pwm.source2
	goto	pwm.source3

pwm.output	bcf	STATUS, RP0	; ensure we're in bank0 after previous block

	movlw	b'00001000'	; LED cathodes drives are active low 
	xorwf	porta.copy,W	; so we invert data before writing to PORTB
	iorwf	led.group,W	; copy in the group transistor drive bit (active high)
	movwf	PORTA	; and output to PORTA

	comf	portb.copy,W	; outputs are active low so we invert data
	movwf	PORTB	; before writing to PORTB

;         Restore context before returning from interrupt
	MOVF    PCLATH_TEMP,W     ; retrieve copy of PCLATH register
          	MOVWF   PCLATH            ; restore pre-isr PCLATH register contents
          	MOVF    STATUS_TEMP,W     ; retrieve copy of STATUS register
          	MOVWF   STATUS            ; restore pre-isr STATUS register contents
          	SWAPF   W_TEMP,F
          	SWAPF   W_TEMP,W          ; restore pre-isr W register contents
          	RETFIE                    ; return from interrupt	



pwm.source1	movfw	pwm.period.count
	addwf	led.red1,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.grn1,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.blu1,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.red4,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.grn4,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.blu4,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.red7,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.grn7,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.blu7,W
	skpnc
	bsf	porta.copy, 3

	goto	pwm.output


pwm.source2	movfw	pwm.period.count
	addwf	led.red2,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.grn2,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.blu2,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.red5,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.grn5,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.blu5,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.red8,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.grn8,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.blu8,W
	skpnc
	bsf	porta.copy, 3

	goto	pwm.output

pwm.source3	movfw	pwm.period.count
	addwf	led.red3,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.grn3,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.blu3,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.red6,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.grn6,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.blu6,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.red9,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.grn9,W
	rrf	portb.copy, f

	movfw	pwm.period.count
	addwf	led.blu9,W
	skpnc
	bsf	porta.copy, 3

	goto	pwm.output







